/* ###################################################################
**     Filename    : Events.c
**     Project     : Test_EthLDDv3
**     Processor   : MK64FN1M0VLL12
**     Component   : Events
**     Version     : Driver 01.00
**     Compiler    : GNU C Compiler
**     Date/Time   : 2015-05-26, 21:41, # CodeGen: 0
**     Abstract    :
**         This is user's event module.
**         Put your event handler code here.
**     Contents    :
**         Cpu_OnNMI - void Cpu_OnNMI(void);
**
** ###################################################################*/
/*!
** @file Events.c
** @version 01.00
** @brief
**         This is user's event module.
**         Put your event handler code here.
*/         
/*!
**  @addtogroup Events_module Events module documentation
**  @{
*/         
/* MODULE Events */

#include "Cpu.h"
#include "Events.h"
#include "Init_Config.h"
#include "PDD_Includes.h"

#ifdef __cplusplus
extern "C" {
#endif 


/* User includes (#include below this line is not maintained by Processor Expert) */
/* User includes (#include below this line is not maintained by Processor Expert) */
#include <stdio.h>
#include "Ethernet.h"

/*
** ===================================================================
**     Event       :  ETH_OnFrameTransmitted (module Events)
**
**     Component   :  ETH [Ethernet_LDD]
**     Description :
**         This event is called after the whole frame on the head of
**         the internal frame queue has been sent and removed from the
**         queue. The event is available only if IEEE 1588 is disabled.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * UserDataPtr     - Pointer to user data
**                           structure pointer.
**     Returns     : Nothing
** ===================================================================
*/
void ETH_OnFrameTransmitted(LDD_TUserData *UserDataPtr)
{
  TEthernetData *EthernetData = (TEthernetData*)UserDataPtr;

  EthernetData->Sent = TRUE;
}

/*
** ===================================================================
**     Event       :  ETH_OnFrameReceived (module Events)
**
**     Component   :  ETH [Ethernet_LDD]
**     Description :
**         This event is called after the whole frame on the head of
**         the internal frame queue has been received and removed from
**         the queue. The event is available only if IEEE 1588 is
**         disabled.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * UserDataPtr     - Pointer to user data
**                           structure pointer.
**         FragCount       - Received frame fragment count
**                           (the number of buffers used to store the
**                           frame data).
**         Length          - Received frame length.
**     Returns     : Nothing
** ===================================================================
*/
void ETH_OnFrameReceived(LDD_TUserData *UserDataPtr, uint16_t FragCount, uint16_t Length)
{
  TEthernetData *EthernetData = (TEthernetData*)UserDataPtr;

  EthernetData->Received += Length;
}

/*
** ===================================================================
**     Event       :  ETH_OnFatalError (module Events)
**
**     Component   :  ETH [Ethernet_LDD]
**     Description :
**         This event is called when a fatal error has occurred (i.e.
**         ethernet bus error). The device should be reinitialized
**         during this event (Deinit and Init methods should be called).
**     Parameters  :
**         NAME            - DESCRIPTION
**       * UserDataPtr     - Pointer to user data
**                           structure pointer.
**     Returns     : Nothing
** ===================================================================
*/
void ETH_OnFatalError(LDD_TUserData *UserDataPtr)
{
  printf("ERROR Fatal ethernet error\n");
  for(;;){}
}
/* END Events */

#ifdef __cplusplus
}  /* extern "C" */
#endif 

/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.5 [05.21]
**     for the Freescale Kinetis series of microcontrollers.
**
** ###################################################################
*/
