/* ###################################################################
**     Filename    : Events.h
**     Project     : Test_EthLDDv3
**     Processor   : MK64FN1M0VLL12
**     Component   : Events
**     Version     : Driver 01.00
**     Compiler    : GNU C Compiler
**     Date/Time   : 2015-05-26, 21:41, # CodeGen: 0
**     Abstract    :
**         This is user's event module.
**         Put your event handler code here.
**     Contents    :
**         Cpu_OnNMI - void Cpu_OnNMI(void);
**
** ###################################################################*/
/*!
** @file Events.h
** @version 01.00
** @brief
**         This is user's event module.
**         Put your event handler code here.
*/         
/*!
**  @addtogroup Events_module Events module documentation
**  @{
*/         

#ifndef __Events_H
#define __Events_H
/* MODULE Events */

#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "Pins1.h"
#include "CsIO1.h"
#include "IO1.h"
#include "ETH.h"
#include "TIMER.h"

#ifdef __cplusplus
extern "C" {
#endif 

void ETH_OnFrameTransmitted(LDD_TUserData *UserDataPtr);
/*
** ===================================================================
**     Event       :  ETH_OnFrameTransmitted (module Events)
**
**     Component   :  ETH [Ethernet_LDD]
**     Description :
**         This event is called after the whole frame on the head of
**         the internal frame queue has been sent and removed from the
**         queue. The event is available only if IEEE 1588 is disabled.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * UserDataPtr     - Pointer to user data
**                           structure pointer.
**     Returns     : Nothing
** ===================================================================
*/

void ETH_OnFrameReceived(LDD_TUserData *UserDataPtr, uint16_t FragCount, uint16_t Length);
/*
** ===================================================================
**     Event       :  ETH_OnFrameReceived (module Events)
**
**     Component   :  ETH [Ethernet_LDD]
**     Description :
**         This event is called after the whole frame on the head of
**         the internal frame queue has been received and removed from
**         the queue. The event is available only if IEEE 1588 is
**         disabled.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * UserDataPtr     - Pointer to user data
**                           structure pointer.
**         FragCount       - Received frame fragment count
**                           (the number of buffers used to store the
**                           frame data).
**         Length          - Received frame length.
**     Returns     : Nothing
** ===================================================================
*/

void ETH_OnFatalError(LDD_TUserData *UserDataPtr);
/*
** ===================================================================
**     Event       :  ETH_OnFatalError (module Events)
**
**     Component   :  ETH [Ethernet_LDD]
**     Description :
**         This event is called when a fatal error has occurred (i.e.
**         ethernet bus error). The device should be reinitialized
**         during this event (Deinit and Init methods should be called).
**     Parameters  :
**         NAME            - DESCRIPTION
**       * UserDataPtr     - Pointer to user data
**                           structure pointer.
**     Returns     : Nothing
** ===================================================================
*/


/* END Events */

#ifdef __cplusplus
}  /* extern "C" */
#endif 

#endif 
/* ifndef __Events_H*/
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.5 [05.21]
**     for the Freescale Kinetis series of microcontrollers.
**
** ###################################################################
*/
