/* ###################################################################
**     Filename    : main.c
**     Project     : Test_EthLDDv3
**     Processor   : MK64FN1M0VLL12
**     Version     : Driver 01.01
**     Compiler    : GNU C Compiler
**     Date/Time   : 2015-05-26, 21:41, # CodeGen: 0
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.01
** @brief
**         Main module.
**         This module contains user's application code.
*/         
/*!
**  @addtogroup main_module main module documentation
**  @{
*/         
/* MODULE main */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "Pins1.h"
#include "CsIO1.h"
#include "IO1.h"
#include "ETH.h"
#include "TIMER.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "PDD_Includes.h"
#include "Init_Config.h"
/* User includes (#include below this line is not maintained by Processor Expert) */

#include <stdio.h>
#include <string.h>
#include "Ethernet.h"

#define MAC_ADDR_SIZE  6                /* MAC address size in bytes */
#define FRAME_LEN_SIZE 2                /* Ethernet frame length size in bytes */
#define CRC_SIZE       4                /* Frame CRC size in bytes */
#define FRAME_HEADER_SIZE (MAC_ADDR_SIZE + MAC_ADDR_SIZE + FRAME_LEN_SIZE)

LDD_TDeviceData *ConsoleUART_DeviceData;

static uint32_t CreateFrame(uint8_t Dest[], uint8_t Source[], uint16_t Length, const uint8_t *Payload, uint8_t *Buffer)
{
  memcpy(&Buffer[0], Dest, MAC_ADDR_SIZE);
  memcpy(&Buffer[6], Source, MAC_ADDR_SIZE);
  Buffer[12] = (Length & 0xFF00U) >> 8;
  Buffer[13] = (Length & 0xFFU);
  memcpy(&Buffer[14], Payload, Length);
  return (FRAME_HEADER_SIZE + Length);
}

/* Align buffer to 16-byte boundary (hardware requirement) */
static uint8_t *AlignBuffer(uint8_t *Ptr)
{
  return (uint8_t*)(((uint32_t)Ptr + 16) & ~0x0FU);
}

static bool Timeout(LDD_TDeviceData *TimerPtr)
{
  return (TIMER_GetEventStatus(TimerPtr) & LDD_TIMERUNIT_ON_COUNTER_RESTART) != 0;
}

void main(void)
{
  LDD_TDeviceData *EthPtr = NULL;
  LDD_TDeviceData *TimerPtr = NULL;
  TEthernetData EthernetData;
  const char *Payload = "Ethernet frame data "
    "0x00 0x01 0x02 0x03 0x04 0x05 0x06 0x07 "
    "0x08 0x09 0x0A 0x0B 0x0C 0x0D 0x0E 0x0F";
  uint8_t MACAddress[] = { 0x06, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
  uint8_t ReceiveBuffer[200];
  uint8_t *AlignedReceiveBuffer = AlignBuffer(ReceiveBuffer);
  uint8_t SendBuffer[200];
  uint8_t *AlignedSendBuffer = AlignBuffer(SendBuffer);
  LDD_ETH_TBufferDesc BufferDesc;
  uint32_t Length;
  LDD_ETH_TStats Stats;

  EthernetData.Sent = FALSE;
  EthernetData.Received = 0;

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  PE_low_level_init();
  /*** End of Processor Expert internal initialization.                    ***/

  //ConsoleUART_DeviceData = ConsoleUART_Init(NULL);

  printf("*********************************************************\r\n");
  printf("*** DEMO project for the Ethernet component - begin      \r\n");
  printf("*********************************************************\r\n");
  printf("\r\n");
  printf("Project description:\r\n");
  printf("Ethernet example using internal loopback with interrupts.\r\n");
  printf("\r\n");

  EthPtr = ETH_Init(&EthernetData);
  TimerPtr = TIMER_Init(NULL);

  printf("+++ Step 0 - begin +++\r\n");
  printf("Description: Send and receive example\r\n");
  memset(ReceiveBuffer, 0, 200);
  memset(SendBuffer, 0, 200);
  printf("1) Initializing reception\r\n");
  ETH_ReceiveFrame(EthPtr, &AlignedReceiveBuffer, 1);
  printf("2) Initializing transmission\r\n");
  printf("  Sending ethernet frame payload: %s\r\n", Payload);
  Length = CreateFrame(MACAddress, MACAddress, strlen(Payload), (const uint8_t*)Payload, AlignedSendBuffer);
  BufferDesc.DataPtr = AlignedSendBuffer;
  BufferDesc.Size = Length;
  ETH_SendFrame(EthPtr, &BufferDesc, 1);
  printf("3) Waiting for the operation to complete\r\n");
  while (!EthernetData.Sent && !Timeout(TimerPtr)) {}
  if (EthernetData.Sent) {
    while ((EthernetData.Received == 0) && !Timeout(TimerPtr)) {}
    if (EthernetData.Received > 0) {
      AlignedReceiveBuffer[EthernetData.Received - CRC_SIZE] = 0;
      printf("  Received ethernet frame payload: %s\r\n", &AlignedReceiveBuffer[FRAME_HEADER_SIZE]);
      if (memcmp(Payload, &AlignedReceiveBuffer[FRAME_HEADER_SIZE], strlen(Payload)) != 0) {
        printf("ERROR Sent and received frame payloads do not match\r\n");
      }
    } else {
      printf("ERROR Receive timed out\r\n");
    }
  } else {
    printf("ERROR Transmission timed out\r\n");
  }
  printf("+++ Step 0 - end +++\r\n");

  printf("\r\n");
  printf("+++ Step 1 - begin +++\r\n");
  printf("Description: Statistics example\r\n");
  Stats = ETH_GetStats(EthPtr);
  printf("1) Ethernet statistics\r\n");
  /* Check statistics items which should be zero */
  if ((Stats.TxRMONDropEvents |
      Stats.TxRMONBroadcastPackets |
      Stats.TxRMONMulticastPackets |
      Stats.TxRMONCRCAlignErrors |
      Stats.TxRMONUndersizePackets |
      Stats.TxRMONOversizePackets |
      Stats.TxRMONFragments |
      Stats.TxRMONJabbers |
      Stats.TxRMONCollisions |
      Stats.TxRMONPackets64Octets |
      Stats.TxRMONPackets128To255Octets |
      Stats.TxRMONPackets256To511Octets |
      Stats.TxRMONPackets512To1023Octets |
      Stats.TxRMONPackets1024To2047Octets |
      Stats.TxRMONPacketsGreaterThan2048Octets |
      Stats.TxIEEEDrop |
      Stats.TxIEEESingleCollision |
      Stats.TxIEEEMultipleCollisions |
      Stats.TxIEEEDeferralDelay |
      Stats.TxIEEELateCollision |
      Stats.TxIEEEExcessiveCollision |
      Stats.TxIEEEFIFOUnderrun |
      Stats.TxIEEECarrierSenseError |
      Stats.TxIEEESQEError |
      Stats.TxIEEEPauseFrame |
      Stats.RxRMONDropEvents |
      Stats.RxRMONBroadcastPackets |
      Stats.RxRMONMulticastPackets |
      Stats.RxRMONCRCAlignErrors |
      Stats.RxRMONUndersizePackets |
      Stats.RxRMONOversizePackets |
      Stats.RxRMONFragments |
      Stats.RxRMONJabbers |
      Stats.RxRMONPackets64Octets |
      Stats.RxRMONPackets128To255Octets |
      Stats.RxRMONPackets256To511Octets |
      Stats.RxRMONPackets512To1023Octets |
      Stats.RxRMONPackets1024To2047Octets |
      Stats.RxRMONPacketsGreaterThan2048Octets |
      Stats.RxIEEEDrop |
      Stats.RxIEEECRCError |
      Stats.RxIEEEAlignmentError |
      Stats.RxIEEEFIFOOverflow |
      Stats.RxIEEEPauseFrame) != 0) {
    printf("ERROR Statistics check for zeroes failed\r\n");
  }
  /* Check other statistics items for correct values */
  if (Stats.TxRMONOctets != (FRAME_HEADER_SIZE + strlen(Payload) + CRC_SIZE)) {
    printf("ERROR ");
  }
  printf("RMON Octet count for frames transmitted without error: %u\r\n", Stats.TxRMONOctets);
  if (Stats.TxRMONPackets != 1) {
    printf("ERROR ");
  }
  printf("RMON Transmitted packet count: %u\r\n", Stats.TxRMONPackets);
  if (Stats.TxRMONPackets65To127Octets != 1) {
    printf("ERROR ");
  }
  printf("RMON Transmitted 65 to 127 byte packets: %u\r\n", Stats.TxRMONPackets65To127Octets);
  if (Stats.TxIEEEFrameOK != 1) {
    printf("ERROR ");
  }
  printf("IEEE Frames transmitted OK: %u\r\n", Stats.TxIEEEFrameOK);
  if (Stats.TxIEEEOctetsOK != (FRAME_HEADER_SIZE + strlen(Payload) + CRC_SIZE)) {
    printf("ERROR ");
  }
  printf("IEEE Octet count for frames transmitted without error: %u\r\n", Stats.TxIEEEOctetsOK);
  if (Stats.RxRMONOctets != (FRAME_HEADER_SIZE + strlen(Payload) + CRC_SIZE)) {
    printf("ERROR ");
  }
  printf("RMON Octet count for frames recieved without error: %u\r\n", Stats.RxRMONOctets);
  if (Stats.RxRMONPackets != 1) {
    printf("ERROR ");
  }
  printf("RMON Received packet count: %u\r\n", Stats.RxRMONPackets);
  if (Stats.RxRMONPackets65To127Octets != 1) {
    printf("ERROR ");
  }
  printf("RMON Received 65 to 127 byte packets: %u\r\n", Stats.RxRMONPackets65To127Octets);
  if (Stats.RxIEEEFrameOK != 1) {
    printf("ERROR ");
  }
  printf("IEEE Frames received OK: %u\r\n", Stats.RxIEEEFrameOK);
  if (Stats.RxIEEEOctetsOK != (FRAME_HEADER_SIZE + strlen(Payload) + CRC_SIZE)) {
    printf("ERROR ");
  }
  printf("IEEE Octet count for frames received without error: %u\r\n", Stats.RxIEEEOctetsOK);
  printf("+++ Step 1 - end +++\r\n");

  ETH_Deinit(EthPtr);
  TIMER_Deinit(TimerPtr);

  printf("\r\n");
  printf("*********************************************************\r\n");
  printf("*** DEMO project for the Ethernet component - end  \r\n");
  printf("*********************************************************\r\n");

  /* Write your code here */
  /* For example: for(;;) { } */

  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;){}
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.5 [05.21]
**     for the Freescale Kinetis series of microcontrollers.
**
** ###################################################################
*/
